/* 
 * Stardust :-) 
 * A Free Space Flight Simulator for a Free Universe!
 * Copyright (C) 2004 MiKael NAVARRO
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdlib.h>

#include <assert.h>

#include <GL/glu.h>

#include "hud.h"

#include "sdlib.h"

#include "vec3d.h"
#include "mat.h"

/*
 * Draw sights (center).
 */
static int hud_draw_sights(Hud *hud) {
  float x, y, z, o, d;

  /* Assertion */
  assert(hud);

  /* Center */
  x = 0.0;
  y = 0.0;
  z = -hud->dz - 0.1;
  /* Size */
  o = hud->w80;
  d = hud->w20;

  /* Draw sights */
  glColor4f(1.0, 0.0, 0.0, 0.8);
  glBegin(GL_POINTS);
  glVertex3f(0.0, 0.0, z);
  glEnd();
  glBegin(GL_LINES);
  glVertex3f(o, 0.0, z);
  glVertex3f(d, 0.0, z);
  glVertex3f(-o, 0.0, z);
  glVertex3f(-d, 0.0, z);
  glVertex3f(0.0, o, z);
  glVertex3f(0.0, d, z);
  glVertex3f(0.0, -o, z);
  glVertex3f(0.0, -d, z);
  glEnd();

  return 0;
}

/*
 * Draw nav controls (center).
 */
static int hud_draw_nav_ctrls(Hud *hud, SpaceShip *player_ship) {
  float x, y, z, o, d;
  float hpr[3];
  GLfloat m[16];

  /* Assertion */
  assert(hud);
  assert(player_ship);

  /* Center */
  x = 0.0;
  y = 0.0;
  z = -hud->dz - 0.1;
  /* Size */
  d = hud->w10;
  o = hud->w40;

  /* Draw nav controls */
  glPushMatrix();
#if 0
  /* Invert player matrix */
  invert_mat4(m, space_ship_mat(player_ship));
  m[12] = m[13] = m[14] = 0; /* reset trans */
  glMultMatrixf(m);
#elif 0 
  /* Invert modelview matrix */
  {
    GLfloat mat[16];

    glGetFloatv(GL_MODELVIEW_MATRIX, mat);
    invert_mat4(m, mat);
    m[12] = m[13] = m[14] = 0; /* reset trans */

    glLoadMatrixf(m);
  }
#else
  /* Use Euler angles */
  {
    GLfloat mat[16];

    invert_mat4(mat, space_ship_mat(player_ship));

    mat4_to_euler(hpr, mat);
    hpr[0] = 0; /* reset rot/y */
    hpr[1] = 180.0-hpr[1];
    hpr[2] = -hpr[2];
    euler_to_mat4(m, hpr);

    glLoadMatrixf(m);
  }
#endif

  glColor4f(1.0, 1.0, 0.0, 0.8);

  {
    float a;
    char str[5];
     
    for(a=0.0; a<=180.0; a+=10.0) {
      glPushMatrix();

      glRotatef(a, 1.0, 0.0, 0.0);
      glBegin(GL_LINES);
      glVertex3f(x-o, y, z);
      glVertex3f(x-d, y, z);
      glVertex3f(x+o, y, z);
      glVertex3f(x+d, y, z);
      glEnd();

      sprintf(str, "%3.0f", a);
#if 0 /* middle */
      glRotatef(hpr[2], 0.0, 0.0, 1.0);
      font_draw(hud->font, str, 8, x-o/2.0, y, z);
#else
      glPushMatrix();
      glTranslatef(d+o, 0.0, 0.0); /* right */
      glRotatef(hpr[2], 0.0, 0.0, 1.0);
      font_draw(hud->font, str, 8, x-o/2.0, y-font_height(hud->font)/2.0, z);
      glPopMatrix();

      glPushMatrix();
      glTranslatef(-d-o, 0.0, 0.0); /* ledt */
      glRotatef(hpr[2], 0.0, 0.0, 1.0);
      font_draw(hud->font, str, 8, x-o/2.0, y-font_height(hud->font)/2.0, z);
      glPopMatrix();
#endif
      glPopMatrix();
    }
    
    glEnable(GL_LINE_STIPPLE);
    glLineStipple(1, 0x7F7F); /*0x1C47*/
    for(a=-10.0; a>-180.0; a-=10.0) {
      glPushMatrix();

      glRotatef(a, 1.0, 0.0, 0.0);
      glBegin(GL_LINES);
      glVertex3f(x-o, y, z);
      glVertex3f(x-d, y, z);
      glVertex3f(x+o, y, z);
      glVertex3f(x+d, y, z);
      glEnd();

      sprintf(str, "%4.0f", a);
#if 0 /* middle */
      glRotatef(hpr[2], 0.0, 0.0, 1.0);
      font_draw(hud->font, str, 8, x-o/2.0, y, z);
#else
      glPushMatrix();
      glTranslatef(d+o, 0.0, 0.0); /* right */
      glRotatef(hpr[2], 0.0, 0.0, 1.0);
      font_draw(hud->font, str, 8, x-o/2.0, y-font_height(hud->font)/2.0, z);
      glPopMatrix();

      glPushMatrix();
      glTranslatef(-d-o, 0.0, 0.0); /* left */
      glRotatef(hpr[2], 0.0, 0.0, 1.0);
      font_draw(hud->font, str, 8, x-o/2.0, y-font_height(hud->font)/2.0, z);
      glPopMatrix();
#endif
      glPopMatrix();
    }
    glDisable(GL_LINE_STIPPLE);
  }

  glPopMatrix();
  
  return 0;
}

/*
 * Draw nav cap (edge middle-up).
 */
static int hud_draw_cap(Hud *hud, SpaceShip *player_ship) {
  /* Assertion */
  assert(hud);
  assert(player_ship);

  /* TODO */

  return 0;
}

/*
 * Display autopilot mode (edge middle-up).
 */
static int hud_draw_autopilot_mode(Hud *hud, SpaceShip *player_ship) {
  float x, y, z, d;

  /* Assertion */
  assert(hud);
  assert(hud->font);
  assert(player_ship);

  /* Edge middle-up */
  x = 0.0;
  y = hud->h2 - hud->unit;
  z = -hud->dz - 0.1;
  /* Size */
  d = 2.0*hud->unit;

  /* Display flying_mode */
  glColor4f(1.0, 1.0, 0.0, 0.8);
  font_draw(hud->font, "S A F M", 8, x+d, y-2.0*d, z);

  return 0;
}

/*
 * Draw NAV controls.
 */
int hud_draw_nav(Hud *hud, SpaceShip *player_ship) {
  /* Assertion */
  assert(hud);
  assert(player_ship);

  /* Draw sights */
  hud_draw_sights(hud);

  if(space_ship_hud_detail(player_ship) >= HUD_HIGH_DETAIL) {
    /* Draw nav controls */
    hud_draw_nav_ctrls(hud, player_ship);
    /* Draw nav cap */
    hud_draw_cap(hud, player_ship);
  }

  /* Display autopilot mode */
  hud_draw_autopilot_mode(hud, player_ship);

  return 0;
}
