/* 
 * Stardust :-) 
 * A Free Space Flight Simulator for a Free Universe!
 * Copyright (C) 2003 MiKael NAVARRO
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * Stardust library.
 * Global definitions, error, warning, debug...
 */

#ifndef _SDLIB_H
#define _SDLIB_H

#include <stdlib.h>
#include <stdio.h>

#include <stdarg.h>

/* Path (datadir) */
#define DATA_DIR DATADIR"/games/"PACKAGE"/"

/* Files, paths length */
#define NAME_LENGTH     (64)
#define FILENAME_LGTH  (512)
#define LINE_LENGTH   (1024)

/* Exit status (0 generally) */
#define SD_SUCCESS EXIT_SUCCESS
#define SD_FAILURE EXIT_FAILURE

/* Level of errors */
#define SD_NONE    0
#define SD_PRINT   1
#define SD_WARNING 2
#define SD_ERROR   3

/*
 * Below some usefull macros to display text,
 * debug informations or warning/error messages.
 */

/*
 * Print an error message on stderr and exit.
 */
void sdf_error(char *fmt, ...);
#define sd_error fprintf(stderr, "%s:%d: ", __FILE__, __LINE__); sdf_error

/*
 * Print a warning message on stderr.
 */
void sdf_warning(char *fmt, ...);
#define sd_warning fprintf(stderr, "%s:%d: ", __FILE__, __LINE__); sdf_warning

/*
 * Print a debug message on stderr.
 */
void sdf_debug(char *fmt, ...);
#ifdef DEBUG
#define sd_debug sdf_warning
#define dprintf sdf_debug
#else
#define sd_debug
#define dprintf
#endif

/*
 * Print a message on stdout.
 */
void sdf_print(char *fmt, ...);
#define printline sdf_print
#define mprintf sdf_print

/* 
 * ifdebug macro,
 * accept several instructions as the "function" parameter.
 */
#ifdef DEBUG
#define ifdebug(function) do { function } while(0)
#else   
#define ifdebug(function) 
#endif  


#endif /* _SDLIB_H */
